import { Component } from '../component';
import { workflows, GithubWorkflow } from '../github';
import { Project } from '../project';
import { Publisher } from '../publisher';
import { Task } from '../tasks';
/**
 * (experimental) Options for `Release`.
 *
 * @experimental
 */
export interface ReleaseProjectOptions {
    /**
     * (experimental) Automatically release new versions every commit to one of branches in `releaseBranches`.
     *
     * @default true
     * @experimental
     */
    readonly releaseEveryCommit?: boolean;
    /**
     * (experimental) CRON schedule to trigger new releases.
     *
     * @default - no scheduled releases
     * @experimental
     */
    readonly releaseSchedule?: string;
    /**
     * (experimental) The name of the main release branch.
     *
     * NOTE: this field is temporarily required as we migrate the default value
     * from "master" to "main". Shortly, it will be made optional with "main" as
     * the default.
     *
     * @default "main"
     * @experimental
     */
    readonly defaultReleaseBranch: string;
    /**
     * (experimental) Branches which trigger a release.
     *
     * Default value is based on defaultReleaseBranch.
     *
     * @default [ "main" ]
     * @experimental
     */
    readonly releaseBranches?: string[];
    /**
     * (experimental) A directory which will contain artifacts to be published to npm.
     *
     * @default "dist"
     * @experimental
     */
    readonly artifactsDirectory?: string;
    /**
     * (experimental) A set of workflow steps to execute in order to setup the workflow container.
     *
     * @experimental
     */
    readonly releaseWorkflowSetupSteps?: workflows.JobStep[];
    /**
     * (experimental) Container image to use for GitHub workflows.
     *
     * @default - default image
     * @experimental
     */
    readonly workflowContainerImage?: string;
    /**
     * (experimental) Version requirement of `jsii-release` which is used to publish modules to npm.
     *
     * @default "latest"
     * @experimental
     */
    readonly jsiiReleaseVersion?: string;
    /**
     * (experimental) The initial version of the repo.
     *
     * The first release will bump over this
     * version, so it will be v0.1.1 or v0.2.0 (depending on whether the first
     * bump is minor or patch).
     *
     * @default "v0.1.0"
     * @experimental
     */
    readonly initialVersion?: string;
    /**
     * (experimental) Bump as a pre-release (e.g. "beta", "alpha", "pre").
     *
     * @default - normal semantic versions
     * @experimental
     */
    readonly prerelease?: string;
    /**
     * (experimental) Steps to execute after build as part of the release workflow.
     *
     * @default []
     * @experimental
     */
    readonly postBuildSteps?: workflows.JobStep[];
    /**
     * (experimental) Checks that after build there are no modified files on git.
     *
     * @default true
     * @experimental
     */
    readonly antitamper?: boolean;
}
/**
 * @experimental
 */
export interface ReleaseOptions extends ReleaseProjectOptions {
    /**
     * (experimental) The task to execute in order to create the release artifacts.
     *
     * Artifacts are
     * expected to reside under `artifactsDirectory` (defaults to `dist/`) once
     * build is complete.
     *
     * @experimental
     */
    readonly task: Task;
    /**
     * (experimental) A name of a .json file to set the `version` field in after a bump.
     *
     * @experimental
     * @example
     *
     * "package.json"
     */
    readonly versionJson: string;
}
/**
 * (experimental) Manages releases (currently through GitHub workflows).
 *
 * @experimental
 */
export declare class Release extends Component {
    /**
     * (experimental) The publisher - responsible for publishing jobs in the workflow.
     *
     * @experimental
     */
    readonly publisher: Publisher;
    /**
     * (experimental) The release workflow.
     *
     * @experimental
     */
    readonly workflow: GithubWorkflow;
    private readonly task;
    private readonly version;
    private readonly postBuildSteps;
    private readonly antitamper;
    private readonly artifactDirectory;
    /**
     * @experimental
     */
    constructor(project: Project, options: ReleaseOptions);
    private createWorkflow;
}
