import { Component } from './component';
import { Project } from './project';
import { Task } from './tasks';
/**
 * (experimental) Options for `Version`.
 *
 * @experimental
 */
export interface VersionOptions {
    /**
     * (experimental) The initial version of the repo.
     *
     * The first release will bump over this
     * version, so it will be v0.1.1 or v0.2.0 (depending on whether the first
     * bump is minor or patch).
     *
     * @default "v0.1.0"
     * @experimental
     */
    readonly initialVersion?: string;
    /**
     * (experimental) A name of a .json file to set the `version` field in after a bump.
     *
     * @experimental
     * @example
     *
     * "package.json"
     */
    readonly versionJson: string;
    /**
     * (experimental) Bump the version as a pre-release tag.
     *
     * @default - normal releases
     * @experimental
     */
    readonly prerelease?: string;
}
/**
 * @experimental
 */
export declare class Version extends Component {
    /**
     * @experimental
     */
    static readonly STANDARD_VERSION = "standard-version@^9";
    /**
     * @experimental
     */
    readonly bumpTask: Task;
    /**
     * @experimental
     */
    readonly unbumpTask: Task;
    /**
     * @experimental
     */
    readonly changelogFile: string;
    /**
     * @experimental
     */
    constructor(project: Project, options: VersionOptions);
}
